<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\Usermanager\tests\integration\Requirements;

use D3\ModCfg\Application\Model\Exception\d3_cfg_mod_exception;
use D3\ModCfg\Application\Model\Exception\d3ShopCompatibilityAdapterException;
use D3\Usermanager\Application\Model\d3usermanager;
use Exception;
use OxidEsales\Eshop\Core\Exception\DatabaseConnectionException;
use OxidEsales\Eshop\Core\Exception\DatabaseErrorException;
use OxidEsales\Eshop\Core\Exception\StandardException;
 
class requirementUserbasketTimespanTest extends d3RequirementIntegrationTestCase
{
    public $sManagerId = 'managerTestId';
    public $aUserIdList = [
        'userTestIdNo1',
        'userTestIdNo2',
        'userTestIdNo3',
        'userTestIdNo4',
        'userTestIdNo5',
        'userTestIdNo6',
    ];
    public $aUserBasketIdList = [
        'userBasketIdNo1',
        'userBasketIdNo2',
        'userBasketIdNo3',
        'userBasketIdNo4',
        'userBasketIdNo5',
    ];

    /**
     * Set up fixture.
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->createTestData();
    }

    /**
     * Tear down fixture.
     */
    public function tearDown(): void
    {
        $this->cleanTestData();

        parent::tearDown();
    }

    /**
     * @throws Exception
     */
    public function createTestData()
    {
        $this->createManager(
            $this->sManagerId
        );

        $this->createUser(
            $this->aUserIdList[0],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[0],
            [
                'oxuserid'    => $this->aUserIdList[0],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d H:i:s')),
            ]
        );

        $this->createUser(
            $this->aUserIdList[1],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[1],
            [
                'oxuserid'    => $this->aUserIdList[1],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d H:i:s') . ' -10 minute'),
            ]
        );

        $this->createUser(
            $this->aUserIdList[2],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[2],
            [
                'oxuserid'    => $this->aUserIdList[2],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d H:i:s') . ' -10 hour'),
            ]
        );

        $this->createUser(
            $this->aUserIdList[3],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[3],
            [
                'oxuserid'    => $this->aUserIdList[3],
                'oxtitle'     => 'savedbasket',
                'oxupdate'     => strtotime(date('Y-m-d H:i:s') . ' -10 day'),
            ]
        );

        $this->createUser(
            $this->aUserIdList[4],
            [
                'oxcompany' => __CLASS__,
            ]
        );

        $this->createBaseModelObject(
            'oxuserbaskets',
            $this->aUserBasketIdList[4],
            [
                'oxuserid'    => $this->aUserIdList[4],
                'oxtitle'     => 'wishlist',
            ]
        );

        $this->createUser(
            $this->aUserIdList[5],
            [
                'oxcompany' => __CLASS__,
            ]
        );
    }

    public function cleanTestData()
    {
        $this->deleteManager($this->sManagerId);

        foreach ($this->aUserIdList as $sUserId) {
            $this->deleteUser($sUserId);
        }

        foreach ($this->aUserBasketIdList as $sUserBasketId) {
            $this->deleteBaseModelObject('oxuserbaskets', $sUserBasketId);
        }
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMaxMinute()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketTimespan_status', true);
        $oManager->setValue('sUserBasketTimespanType', 'maximum');
        $oManager->setValue('sUserBasketTimespanUnit', 'minute');
        $oManager->setValue('sUserBasketTimespanValue', 5);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMaxMinute()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMaxMinute());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 1
            && $oUserList->offsetExists($this->aUserIdList[0])
            && false == $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
            && false == $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
            && false == $oUserList->offsetExists($this->aUserIdList[5])
        );
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMaxHour()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketTimespan_status', true);
        $oManager->setValue('sUserBasketTimespanType', 'maximum');
        $oManager->setValue('sUserBasketTimespanUnit', 'hour');
        $oManager->setValue('sUserBasketTimespanValue', 5);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMaxHour()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMaxHour());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 2
            && $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
            && false == $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
            && false == $oUserList->offsetExists($this->aUserIdList[5])
        );
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMaxDay()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketTimespan_status', true);
        $oManager->setValue('sUserBasketTimespanType', 'maximum');
        $oManager->setValue('sUserBasketTimespanUnit', 'day');
        $oManager->setValue('sUserBasketTimespanValue', 5);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMaxDay()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMaxDay());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 3
            && $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && $oUserList->offsetExists($this->aUserIdList[2])
            && false == $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
            && false == $oUserList->offsetExists($this->aUserIdList[5])
        );
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMinMinute()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketTimespan_status', true);
        $oManager->setValue('sUserBasketTimespanType', 'minimum');
        $oManager->setValue('sUserBasketTimespanUnit', 'minute');
        $oManager->setValue('sUserBasketTimespanValue', 5);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMinMinute()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMinMinute());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 3
            && false == $oUserList->offsetExists($this->aUserIdList[0])
            && $oUserList->offsetExists($this->aUserIdList[1])
            && $oUserList->offsetExists($this->aUserIdList[2])
            && $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
            && false == $oUserList->offsetExists($this->aUserIdList[5])
        );
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMinHour()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketTimespan_status', true);
        $oManager->setValue('sUserBasketTimespanType', 'minimum');
        $oManager->setValue('sUserBasketTimespanUnit', 'hour');
        $oManager->setValue('sUserBasketTimespanValue', 5);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMinHour()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMinHour());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 2
            && false == $oUserList->offsetExists($this->aUserIdList[0])
            && false == $oUserList->offsetExists($this->aUserIdList[1])
            && $oUserList->offsetExists($this->aUserIdList[2])
            && $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
            && false == $oUserList->offsetExists($this->aUserIdList[5])
        );
    }

    /**
     * @return d3usermanager
     * @throws Exception
     */
    public function getConfiguredManagerMinDay()
    {
        $oManager = $this->getManagerMock($this->sManagerId);

        $oManager->setValue('blCheckUserBasketTimespan_status', true);
        $oManager->setValue('sUserBasketTimespanType', 'minimum');
        $oManager->setValue('sUserBasketTimespanUnit', 'day');
        $oManager->setValue('sUserBasketTimespanValue', 5);

        return $oManager;
    }

    /**
     * @test
     * @throws DatabaseConnectionException
     * @throws DatabaseErrorException
     * @throws StandardException
     * @throws d3ShopCompatibilityAdapterException
     * @throws d3_cfg_mod_exception
     * @throws Exception
     */
    public function requirementsSelectsRightUsersMinDay()
    {
        $oListGenerator = $this->getListGenerator($this->getConfiguredManagerMinDay());
        $oUserList = $oListGenerator->getConcernedItems();

        $this->assertTrue(
            $oUserList->count() >= 1
            && false == $oUserList->offsetExists($this->aUserIdList[0])
            && false == $oUserList->offsetExists($this->aUserIdList[1])
            && false == $oUserList->offsetExists($this->aUserIdList[2])
            && $oUserList->offsetExists($this->aUserIdList[3])
            && false == $oUserList->offsetExists($this->aUserIdList[4])
            && false == $oUserList->offsetExists($this->aUserIdList[5])
        );
    }
}
